=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-08-25 03:20:44 +0000
+++ bzrlib/builtins.py	2011-08-29 18:20:22 +0000
@@ -1341,18 +1341,34 @@
 class cmd_branches(Command):
     __doc__ = """List the branches available at the current location.
 
-    This command will print the names of all the branches at the current location.
+    This command will print the names of all the branches at the current
+    location.
     """
 
     takes_args = ['location?']
+    takes_options = [
+                  Option('scan', help='Recursively scan for for branches '
+                                      'rather than just looking in the '
+                                      'specified location.')]
 
-    def run(self, location="."):
-        dir = bzrdir.BzrDir.open_containing(location)[0]
-        for branch in dir.list_branches():
-            if branch.name is None:
-                self.outf.write(" (default)\n")
-            else:
-                self.outf.write(" %s\n" % branch.name.encode(self.outf.encoding))
+    def run(self, location=".", scan=False):
+        if scan:
+            t = transport.get_transport(location)
+            if not t.listable():
+                raise errors.BzrCommandError(
+                    "Can't scan this type of location.")
+            for b in bzrdir.BzrDir.find_branches(t):
+                self.outf.write("%s\n" % urlutils.unescape_for_display(
+                    urlutils.relative_url(t.base, b.base),
+                    self.outf.encoding).rstrip("/"))
+        else:
+            dir = bzrdir.BzrDir.open_containing(location)[0]
+            for branch in dir.list_branches():
+                if branch.name is None:
+                    self.outf.write(" (default)\n")
+                else:
+                    self.outf.write(" %s\n" % branch.name.encode(
+                        self.outf.encoding))
 
 
 class cmd_checkout(Command):

=== modified file 'bzrlib/tests/blackbox/test_branches.py'
--- bzrlib/tests/blackbox/test_branches.py	2011-08-15 15:48:12 +0000
+++ bzrlib/tests/blackbox/test_branches.py	2011-08-29 18:20:22 +0000
@@ -41,3 +41,16 @@
         self.run_bzr('init-repo a')
         out, err = self.run_bzr('branches', working_dir='a')
         self.assertEquals(out, "")
+
+    def test_scan(self):
+        self.run_bzr('init source')
+        self.run_bzr('init source/subsource')
+        self.run_bzr('checkout --lightweight source checkout')
+        self.run_bzr('init checkout/subcheckout')
+        self.run_bzr('init checkout/.bzr/subcheckout')
+        out = self.run_bzr('branches --scan')[0]
+        lines = out.split('\n')
+        self.assertIs(True, 'source' in lines, lines)
+        self.assertIs(True, 'source/subsource' in lines, lines)
+        self.assertIs(True, 'checkout/subcheckout' in lines, lines)
+        self.assertIs(True, 'checkout' not in lines, lines)

